#include "gskutils.h"
#include <string.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <errno.h>
#include <ctype.h>
#include <fcntl.h>
#include "gskerror.h"

gboolean gsk_mkdir_p (const char *dir,
                      guint       permissions,
		      GError    **error)
{
  guint dir_len = strlen (dir);
  char *dir_buf = g_alloca (dir_len + 1);
  guint cur_len = 0;

  /* append to dir_buf any number of consecutive '/'
     characters. */
#define SCAN_THROUGH_SLASHES()  \
  G_STMT_START{                 \
    while (cur_len < dir_len && dir[cur_len] == G_DIR_SEPARATOR) \
      dir_buf[cur_len++] = G_DIR_SEPARATOR; \
  }G_STMT_END

  SCAN_THROUGH_SLASHES();
  while (cur_len < dir_len)
    {
      const char *slash = strchr (dir + cur_len, G_DIR_SEPARATOR);    /* not UTF8 */
      guint new_cur_len;
      if (slash == NULL)
        new_cur_len = dir_len;
      else
        new_cur_len = slash - dir;

      memcpy (dir_buf + cur_len, dir + cur_len, new_cur_len - cur_len);
      dir_buf[new_cur_len] = 0;
      cur_len = new_cur_len;

      if (g_file_test (dir_buf, G_FILE_TEST_IS_DIR))
        ;
      else
        {
          if (mkdir (dir_buf, permissions) < 0)
            {
              g_set_error (error, GSK_G_ERROR_DOMAIN,
                           gsk_error_code_from_errno (errno),
                           "error making directory %s: %s",
                           dir_buf, g_strerror (errno));
              return FALSE;
            }
        }

      SCAN_THROUGH_SLASHES();
    }
  return TRUE;
#undef SCAN_THROUGH_SLASHES
}

char *
gsk_escape_memory (gconstpointer    data,
		   guint            len)
{
  GString *out = g_string_new ("");
  guint i;
  for (i = 0; i < len; i++)
    {
      guint8 c = ((guint8 *) data)[i];
      if (!isprint (c) || c <= 27 || c == '"' || c == '\\')
	{
	  switch (c)
	    {
	    case '\t':
	      g_string_append (out, "\\t");
	      break;
	    case '\r':
	      g_string_append (out, "\\r");
	      break;
	    case '\n':
	      g_string_append (out, "\\n");
	      break;
	    case '\\':
	      g_string_append (out, "\\\\");
	      break;
	    case '"':
	      g_string_append (out, "\\\"");
	      break;
	    default:
	      g_string_sprintfa (out, "\\%o", c);
	      break;
	    }
	}
      else
	{
	  g_string_append_c (out, c);
	}
    }
  return g_string_free (out, FALSE);
}

void  gsk_fd_set_close_on_exec (int fd, gboolean close_on_exec)
{
  int fdflags = fcntl (fd, F_GETFD);
  if (close_on_exec)
    fdflags |= FD_CLOEXEC;
  else
    fdflags &= ~FD_CLOEXEC;
  fcntl (fd, F_SETFD, fdflags);
}
